@@ -26,6 +26,7 @@ import serial
 from .util import *
 from .repo import *
 from serial.tools import miniterm
+from serial.tools.list_ports import comports
 
 def get_host_os():
     host_os = platform.system()
@@ -43,6 +44,15 @@ def get_host_os():
         host_os = None
     return host_os
 
+def get_aos_url():
+    """Figure out proper URL for downloading AliOS-Things."""
+    if is_domestic():
+        aos_url = 'https://gitee.com/alios-things/AliOS-Things.git'
+    else:
+        aos_url = 'https://github.com/alibaba/AliOS-Things.git'
+
+    return aos_url
+
 def cd_aos_root():
     original_dir = os.getcwd()
     host_os = get_host_os()
@@ -50,7 +60,7 @@ def cd_aos_root():
         sys_root = re.compile(r'^[A-Z]{1}:\\$')
     else:
         sys_root = re.compile('^/$')
-    while os.path.isdir('./include/aos') == False:
+    while os.path.isdir('./kernel/rhino') == False:
         os.chdir('../')
         if sys_root.match(os.getcwd()):
             return 'fail', original_dir
@@ -366,6 +376,80 @@ def subcommand(name, *args, **kwargs):
     return __subcommand
 
 
+# Sub command: setup
+@subcommand('setup',
+            dict(name=['-d', '--dest-dir'], dest='dest_dir',
+                 help='Destination dir'),
+            dict(name=['-b', '--branch'], dest='branch',
+                 help='Branch to checkout'),
+            dict(name=['-m', '--board'], dest='board',
+                 help='Board name'),
+            dict(name=['-s', '--strip-sources'], dest='strip_sources', action='store_true',
+                 help='Strip irrelevant part from sources: git history, other boards ...'),
+            help='Setup AliOS Things sources tree (experimental)',
+            description=(
+                    "Setup AliOS Things sources tree\n"
+                    ))
+def setup(dest_dir='./', branch='master', board=None, strip_sources=False):
+    """ Download sources to specific directory, allow to strip irrelevant part for board building. """
+
+    source_dir = os.path.join(dest_dir, OS_NAME)
+    if source_dir and os.path.isdir(os.path.join(source_dir, '.git')):
+        action('AliOS-Things code has been download \"%s\"' % source_dir)
+    else:
+        aos_url = get_aos_url()
+        aos_url += '#' + branch
+        try:
+            add(aos_url, path=source_dir)
+        except Exception as e:
+            raise e
+
+    if board and strip_sources:
+        action('Board %s specified with --strip-sources:' % board)
+        _strip_sources(source_dir, board)
+
+def _strip_sources(source_dir, board):
+    """ Remove .git/, board/*, platform/mcu/* dirs from sources. """
+
+    strip_list = ['/.git', '/.aos']
+    board_arch = None
+
+    # Get board arch family from board's makefile
+    board_mk = os.path.join(source_dir, 'board', board, board + '.mk')
+    if os.path.isfile(board_mk):
+        with open(board_mk) as file:
+            for board_arch in file.readlines():
+                if 'HOST_MCU_FAMILY' in board_arch:
+                    break
+
+        regex_board_arch = r'^HOST_MCU_FAMILY\s*:?=\s*(.*)'
+        match = re.match(regex_board_arch, board_arch)
+        board_arch = match.group(1)
+    else:
+        error('Unknow board name %s!' % board)
+
+    mcu_list = os.listdir(source_dir + '/platform/mcu')
+    mcu_list.remove('include')
+    mcu_list.remove(board_arch)
+    mcu_list = [ os.path.join('/platform/mcu/', item) for item in mcu_list ]
+
+    board_list = os.listdir(source_dir + '/board')
+    board_list.remove(board)
+    board_list = [ os.path.join('/board/', item) for item in board_list ]
+
+    strip_list += mcu_list + board_list
+
+    for item in strip_list:
+        tmp_dir = os.path.join(source_dir + item)
+        if os.path.isdir(tmp_dir):
+            action('Removing dir: %s' % tmp_dir)
+            shutil.rmtree(tmp_dir)
+        elif os.path.isfile(tmp_dir):
+            action('Removing file: %s' % tmp_dir)
+            os.remove(tmp_dir)
+        else:
+            pass
+
 # New command
 @subcommand('new',
             dict(name='name', help='The program or component name'),
@@ -415,10 +499,7 @@ def new(name, scm='git', component=False, template=None, protocol=None):
     if d_type == 'program':
         # This helps sub-commands to display relative paths to the created program
         # p.set_root()
-        if is_domestic():
-            aos_os_url = 'https://gitee.com/alios-things/AliOS-Things.git'
-        else:
-            aos_os_url = 'https://github.com/alibaba/AliOS-Things.git'
+        aos_os_url = get_aos_url()
         url = aos_os_url + '#master'
         d = 'aos'
         aos_sdk_path = Global().get_cfg(AOS_SDK_PATH)
@@ -1198,14 +1279,17 @@ def make_build(make_args):
                 if build_dir:
                     shutil.rmtree(out_path, True)
                 pd.set_cfg(CUBE_MODIFY, '0')
-
             _run_make(['-e', '-f build/Makefile', make_args, app_dir, build_dir])
     else:
         # aos source code
         _run_make(['-e', '-f build/Makefile', make_args])
 
 def scons_build(args):
-    make_args = ['scons -j4 -f ucube.py']
+    if os.path.exists('ucube.py') == True:
+        make_args = ['scons -j4 -f ucube.py']
+    else:
+        make_args = ['scons -j4 -f build/ucube.py']
+        
     target_find = False
 
     for arg in args:
@@ -1224,6 +1308,9 @@ def scons_build(args):
                 make_args.append(build_type)
 
             target_find = True
+        elif arg.startswith('JOBS=') and arg.replace('JOBS=', '').isdigit() == True:
+            jnum = arg.replace('JOBS=', '-j')
+            make_args[0] = make_args[0].replace('-j4', jnum)
         else:
             make_args.append(arg)
 
@@ -1244,6 +1331,9 @@ def get_scons_enabled_boards():
             help='Make aos program/component',
             description="Make aos program/component.")
 def make():
+    from .constant import ver
+    log('aos-cube version: %s\n' %ver)
+
     # Get the make arguments
     args = sys.argv[2:]
 
@@ -1383,157 +1473,43 @@ def config_(var=None, value=None, global_cfg=False, unset=False, list_config=Fal
     else:
         subcommands['config'].error("too few arguments")
 
-def upload_mk3060(target, aos_path):
-    host_os = get_host_os()
-    if host_os == None:
-        error('Unsupported Operating System!')
-    cmd = os.path.join(aos_path, 'build', 'aos_firmware_update.py').replace('\\', '/')
-    device_port = miniterm.ask_for_port()
-    log('\n[INFO]:cmd: %s\n' % cmd)
-    addr = '0x13200'
-    image_path = os.path.join(os.getcwd(), 'out', target, 'binary', target + '.ota.bin').replace('\\', '/')
-    log("\n[INFO]:image_path: '%s'\n" % image_path)
+def scons_upload(args):
+    make_args = ['scons -f ucube.py COMMAND=upload']
+    target_find = False
 
-    exec_cmd = [
-    cmd,
-    device_port, #/dev/ttyUSBx
-    addr,
-    image_path
-    ];
-    log("\n[INFO]:exec_cmd: %s\n" % exec_cmd)
-    if (host_os == 'Win32'):
-        ret = subprocess.call(exec_cmd, shell=True)
-        log("---host_os:%s\n" % host_os)
-    else:
-        ret = subprocess.call(exec_cmd, stdout=sys.stdout, stderr=sys.stderr)
-        log("---host_os:%s\n" % host_os)
-    return ret
+    for arg in args:
+        if '@' in arg and not target_find:
+            targets = arg.split('@')
+            if len(targets) < 2:
+                error('Must special app and board when build aos')
 
-def upload_esp32(target, aos_path):
-    host_os = get_host_os()
-    if host_os == None:
-        error('Unsupported Operating System!')
-    cmd = 'esptool.py'
-    port = miniterm.ask_for_port()
-    image_path = os.path.join(os.getcwd(), 'out', target, 'binary', target + '.bin').replace('\\', '/')
-    bootloader_path = os.path.join(aos_path, 'platform/mcu/esp32/bsp/bootloader.bin').replace('\\', '/')
-    partition_path = os.path.join(aos_path, 'platform/mcu/esp32/bsp/custom_partitions.bin').replace('\\', '/')
-    log("\n[INFO]:image_path: %s\n" % image_path)
-    log("[INFO]:bootloader_path: %s\n" % bootloader_path)
-    log("[INFO]:partition_path: %s\n" % partition_path)
+            app = 'APPLICATION='+targets[0]
+            board = 'BOARD=' + targets[1]
+            make_args.append(app)
+            make_args.append(board)
 
-    exec_cmd = [
-	cmd,
-	'--chip',
-	'esp32',
-	'--port', port,
-	'--baud', '921600',
-	'--before', 'default_reset',
-	'--after', 'hard_reset',
-	'write_flash',
-	'-z',
-	'--flash_mode',
-	'dio',
-	'--flash_freq',
-	'40m',
-	'--flash_size',
-	'detect',
-	'0x1000', bootloader_path,
-	'0x8000', partition_path,
-	'0x10000', image_path
-    ]
-    log("\n[INFO]:exec_cmd: %s\n" % exec_cmd)
-    if (host_os == 'Win32'):
-        ret = subprocess.call(exec_cmd, shell=True)
-        log("---host_os:%s\n" % host_os)
-    else:
-        ret = subprocess.call(exec_cmd, stdout=sys.stdout, stderr=sys.stderr)
-        log("---host_os:%s\n" % host_os)
-    return ret
+            if len(targets) == 3:
+                build_type = 'TYPE=' + targets[2]
+                make_args.append(build_type)
 
-def upload_esp8266(target, aos_path):
-    host_os = get_host_os()
-    if host_os == None:
-        error('Unsupported Operating System!')
-    cmd = 'esptool.py'
-    port = miniterm.ask_for_port()
-    image_path = os.path.join(os.getcwd(), 'out', target, 'binary', target + '-0x1000.bin').replace('\\', '/')
-    boot_bin_path = os.path.join(aos_path, 'platform/mcu/esp8266/bsp/boot_v1.7_921600.bin').replace('\\', '/')
-    init_bin_path = os.path.join(aos_path, 'platform/mcu/esp8266/bsp/esp_init_data_default.bin').replace('\\', '/')
-    blank_bin_path = os.path.join(aos_path, 'platform/mcu/esp8266/bsp/blank.bin').replace('\\', '/')
+            target_find = True
+        #else:
+        #    ret = make_args.append(arg)
 
-    exec_cmd = [
-	cmd,
-	'--chip', 'esp8266',
-	'--port', port,
-	'--baud', '921600',
-	'write_flash',
-	'--flash_size',
-	'detect',
-	'0x0', boot_bin_path,
-	'0x3fc000', init_bin_path,
-	'0x3fe000', blank_bin_path,
-	'0x1000', image_path
-    ]
-    log("\n[INFO]:exec_cmd: %s\n" % exec_cmd)
-    if (host_os == 'Win32'):
-        ret = subprocess.call(exec_cmd, shell=True)
-        log("---host_os:%s\n" % host_os)
-    else:
-        ret = subprocess.call(exec_cmd, stdout=sys.stdout, stderr=sys.stderr)
-        log("---host_os:%s\n" % host_os)
-    return ret
-
-def upload_stm32(target, aos_path):
-    upload_cmds = {
-        'Win32': 'win32/st-flash.exe',
-        'Linux64': 'linux64/st-flash',
-        'Linux32': 'linux32/st-flash',
-        'OSX': 'osx/st-flash'
-    }
-    board = target.split('@')[1]
-    host_os = get_host_os()
-    if host_os == None:
-        error('Unsupported Operating System!')
-
-    cmds_dir = os.path.join(aos_path, 'build', 'cmd').replace('\\', '/')
-    upload_cmd = os.path.join(cmds_dir, upload_cmds[host_os]).replace('\\', '/')
-    log("\n[INFO]:upload_cmd: '%s'\n" % upload_cmd)
-    addr = '0x08000000'
-
-    if 'starterkit' in board:
-	    # remove unnecessay erase step
-        erase_cmd = [upload_cmd, 'erase']
-
-        ret = subprocess.call(erase_cmd, stdout=sys.stdout, stderr=sys.stderr)
-        if ret == 0:
-            print('erase %s succeed' % board)
-        else:
-            print('erase %s failed' % board)
-
-    image_path = os.path.join(os.getcwd(), 'out', target, 'binary', target + '.bin')
-    log("\n[INFO]:image_path: '%s'\n" % image_path)
-
-    exec_cmd = [upload_cmd, '--reset', 'write', image_path, addr];
-
-    ret = subprocess.call(exec_cmd, stdout=sys.stdout, stderr=sys.stderr)
+    ret = popen(' '.join(make_args), shell=True, cwd=os.getcwd())
+#    ret = subprocess.call(make_args, stdout=sys.stdout, stderr=sys.stderr)
     return ret
 
 def _run_upload(target, aos_path):
-    app = target.split('@')[0]
-    board = target.split('@')[1]
-    ret = None
-
-    if 'esp32' in board:
-        ret = upload_esp32(target, aos_path)
-    elif 'mk3060' in board:
-        ret = upload_mk3060(target, aos_path)
-    elif 'esp8266' in board:
-        ret = upload_esp8266(target, aos_path)
-    else:
-        ret = upload_stm32(target, aos_path)
+    from .constant import ver
+    log('aos-cube version: %s\n' %ver)
+    # Get the make arguments
+    args = sys.argv[2:]
+    ret = scons_upload(args)
     return ret
 
+
+
 # upload command
 @subcommand('upload',
             dict(name='target', help='aos target, e.g. helloworld@starterkit'),
@@ -1541,7 +1517,9 @@ def _run_upload(target, aos_path):
             description="Upload aos image to target platform.")
 
 def upload(target):
-    """ FIXME we only support stlink tools for now """
+    from .constant import ver
+    log('aos-cube version: %s\n' %ver)
+
     log("[INFO]:target: %s\n" % target)
 
     if os.path.isdir('./include/aos') == False:
@@ -1559,12 +1537,8 @@ def upload(target):
         error('target invalid')
         return;
 
-    ret = _run_upload(target, aos_path)
+    _run_upload(target, aos_path)
 
-    if ret == 0:
-        print('firmware upload succeed')
-    else:
-        print('firmware upload failed')
 
 
 @subcommand('monitor',
@@ -1584,6 +1558,17 @@ def monitor(port, baud):
     except ProcessException as e:
         raise e
 
+@subcommand('devices',
+            help='List devices on serial ports\n\n',
+            description="List devices on serial ports.")
+def devices():
+    arr = []
+    for p in comports():
+        j = json.dumps(p.__dict__)
+        arr.append(json.loads(j))
+    print(json.dumps(arr, indent = 4))
+    sys.exit(0)
+
 @subcommand('upgrade',
             help='Upgrade aos-cube to latest',
             description="Run pip upgrade process to keep aos-cube up-to-date.")
